# -*- coding: utf-8 -*-

import os
import logging

from gnuradio import blocks
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from optparse import OptionParser
import osmosdr

from PyQt5.QtCore import QThread, pyqtSignal, QObject

from lib.disk import Disk

class FileSink(gr.top_block):
    """
    FileSink class to create I/Q sample binary dump files.
    """
    def __init__(self,
                 samp_rate,
                 freq,
                 capture_time,
                 dump_file,
                 device=None
    ):
        gr.top_block.__init__(self, "file_sink_block")

        self.samp_rate = samp_rate
        self.freq = freq
        self.capture_time = capture_time
        self.file_path = dump_file
        self.device = osmosdr.source( args="numchan=" + str(1) + " " + "osmosdr=0" )


    def needed_disc_space(self):
        """
        Calculate needed disc space for dump.
        """
        pass

    def run(self):
        """
        Capture alias
        """
        # self.capture()

    def capture(self):
        """
        Capture samples to dump file.
        """
        self._build_blocks()
        logging.info("Start '%s' capture." % self.file_path)
        self.start()
        self.wait()
        logging.info("Capture '%s' done." % self.file_path)


    def get_samp_rate(self):
        return self.samp_rate

    def get_capture_time(self):
        return self.capture_time

    def get_capture_device(self):
        return self.device

    def _dump_file_writable(self):
        """
        Check permissions for specified file dump path.
        """
        Disk.path_writeble(self.file_path)

    def _build_blocks(self):
        """
        Create gr blocks.
        """
        # blocks
        self.osmosdr_source_0 = self.device
        self.osmosdr_source_0.set_sample_rate(self.samp_rate)
        self.osmosdr_source_0.set_center_freq(self.freq, 0)
        self.osmosdr_source_0.set_freq_corr(0, 0)
        self.osmosdr_source_0.set_dc_offset_mode(0, 0)
        self.osmosdr_source_0.set_iq_balance_mode(0, 0)
        self.osmosdr_source_0.set_gain_mode(False, 0)
        self.osmosdr_source_0.set_gain(0, 0)
        self.osmosdr_source_0.set_if_gain(20, 0)
        self.osmosdr_source_0.set_bb_gain(20, 0)
        self.osmosdr_source_0.set_antenna("", 0)
        self.osmosdr_source_0.set_bandwidth(0, 0)

        self.blocks_head_0 = blocks.head(gr.sizeof_gr_complex*1, int(self.samp_rate)*self.capture_time)
        self.blocks_file_sink_0 = blocks.file_sink(gr.sizeof_gr_complex*1, str(self.file_path), False)
        self.blocks_file_sink_0.set_unbuffered(False)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_head_0, 0), (self.blocks_file_sink_0, 0))
        self.connect((self.osmosdr_source_0, 0), (self.blocks_head_0, 0))